package tunneler;

import java.awt.Color;
/**
 * @author vondrm4
 *
 *
 * obsahuje pole Bod reprezentujici mapu hry, taktez obsahuje barvy
 * pro vykreslovani strel, hliny, kamen
 */
public class Mapa {

    protected class Bod{
        int typ;
        Color barva;
        Hrac parent;
        protected Bod(int typ, Color barva){
            this.typ=typ;
            this.parent=null;
            this.barva=barva;
        }
        protected Bod(int typ, Hrac parent){
            this.typ=typ;
            this.parent=parent;
            this.barva=parent.barva;
        }
		protected void setValues(int typ, Color barva, Hrac parent){
			this.typ=typ;
			this.barva=barva;
			this.parent=parent;
		}
    }

    //system [x][y]
    Bod[][] mapa;
    int sirka;
    int vyska;
    Color[] barvaHliny;
    Color volnyProstor;
    Color[] strela=new Color[2];
    Color energy;
    Color shield;
    Color kamen;
    Color menu;
    Color load;
	Color kanon;

	//typy materialu
	// TANK mus mit v Bodu hodnotu parent!!!
	final int NEPROSTUPNE=0;
	final int VOLNE=1;
	final int HLINA=2;
	final int TANK=3;
	final int STRELA=4;

    //hodnoty pro generovani zakladen
    int zakSirka;
    int zakVyska;
	int[][] hraciStart;

    public Mapa(int sirka,int vyska) {
		this.sirka=sirka;
		this.vyska=vyska;

		barvaHliny=new Color[4];
		barvaHliny[0]=new Color(135,117,93);
		barvaHliny[1]=new Color(125,97,73);
		barvaHliny[2]=new Color(135,87,73);
		barvaHliny[3]=new Color(115,87,83);

		volnyProstor=Color.BLACK;
		strela[0]=Color.RED;
		strela[1]=new Color(150,0,0);

		energy=Color.YELLOW;
		shield=new Color(100,100,255);

		zakSirka=20;
		zakVyska=25;

		kanon=Color.YELLOW;

		kamen=Color.GRAY;
		menu=new Color(90,90,90);
		load=Color.GRAY;

		mapa=generujZem();
		//nutne generovat jako prvni aby byla se Hraci a pod pohled mohly inicializovat

    }
	public int[][] generovat(Color[] barvy){
		int[][] polohyBase=new int[barvy.length][2];

		int[] nahodneGeneruj=nahodneRozhodit(barvy.length);
		for (int i = 0; i < nahodneGeneruj.length; i++) {
			polohyBase[nahodneGeneruj[i]] = setBase(barvy[nahodneGeneruj[i]], i);

		}
		return polohyBase;
	}
    private Bod[][] generujZem(){
        Bod[][] list=new Bod[sirka][vyska];
        Bod prvek;
		for(int i=0; i<(sirka);i++){
			for(int l=0; l<(vyska);l++){
				if(i==0 || i==sirka-1 || i==sirka-1 || (l==0 || l==vyska-1 )){
					prvek=drawKamen();
				}else{
					prvek=drawHlina();
				}
				list[i][l]=prvek;
			}
		}

        return list;
    }
	private  int[] nahodneRozhodit(int pocet){
	//zprehazeni prvku pole pro nahodne rozmisteni zakladen po mape
		int[] nahoda=new int[pocet];
		int[] prvky=new int[pocet];
		//naplneni indexi
		for (int i = 0; i < pocet; i++) {
			prvky[i]=i;
		}
		int random;
		for (int i = 0; i < nahoda.length; i++) {
			random=(int) Math.round(Math.random()*(prvky.length-1));
			nahoda[i] = prvky[random];
			prvky=resizePole(prvky, random);
		}

		return nahoda;
	}
	private int[] resizePole(int[] poleIn, int prvek){
		int[] poleOut=new int[poleIn.length-1];
		int vnitrniIterator=0;
		for (int i = 0; i < poleIn.length; i++) {
			if(i!=prvek) {
				poleOut[vnitrniIterator] = poleIn[i];
				vnitrniIterator ++;
			}
		}
		return poleOut;
	}
	private Bod drawKamen(){
		return new Bod(NEPROSTUPNE,kamen);
	}
	private Bod drawHlina(){
		return new Bod(HLINA, nahBarva());
	}
	private Bod drawEmpty(){
		return new Bod(VOLNE, volnyProstor);
	}
	private Bod drawZakl(Color barva){
		return new Bod(NEPROSTUPNE, barva);
	}
	public void redrawTankBod(Hrac parent, int x,int y){
		mapa[x][y].setValues(TANK, parent.barva, parent);
	}
	public void redrawTankKanon(Hrac parent, int x, int y){
		mapa[x][y].setValues(TANK, kanon, parent);
	}

	public void redrawStrela(int x, int y, int polX, int polY){
		mapa[x][y].setValues(STRELA, strela[0], null);
		mapa[x+polX][y+polY].setValues(STRELA, strela[1], null);
	}
	public void redrawStrelaEmpty(int x, int y, int polX, int polY){
		mapa[x][y].setValues(VOLNE, volnyProstor, null);
		mapa[x+polX][y+polY].setValues(VOLNE, volnyProstor, null);
	}
	public void redrawEmpty(int x, int y){
		mapa[x][y].setValues(VOLNE, volnyProstor, null);
	}

    private int[] setBase(Color barva2, int p){
		//vykresli v mape zakladnu a vrati jeji pozici pro hrace,
		//p urcuje v jake casti se ma vykreslit
		int[] vrat=new int[2];
	    int polX=0;
	    int polY=0;

		if(p==0 || p==2){
			polX=((int) Math.round(Math.random()*((sirka/3)-zakSirka)))+zakSirka/2;
			
		}
		if(p==1 || p==3){
			polX=((int) Math.round(Math.random()*((sirka/3)-zakSirka)))+zakSirka/2+sirka/2;
		}
		if(p==0 || p==1){
			polY=((int) Math.round(Math.random()*(vyska/2-zakVyska-12)))+6;
		}
		if(p==2 || p==3){
			polY=((int) Math.round(Math.random()*(vyska/2-zakVyska-12)))+6+ vyska/2;
		}


	    for(int i=0; i<zakSirka;i++){
			for(int l=0; l<zakVyska;l++){
				if(i==0 || i==zakSirka-1 || ((l==0 || l==zakVyska-1)&&(i<8||i>12)))
					mapa[polX+i][polY+l]=drawZakl(barva2);
				else mapa[polX+i][polY+l]=drawEmpty();

			}
	    }
	    vrat[0]=polX;
	    vrat[1]=polY;
	    return vrat;

    }

    private Color nahBarva(){
		int nahoda=(int) Math.round(Math.random()*3);
		return barvaHliny[nahoda];
    }
    
    public Color[][] getPodPole(int x, int y, int sirka, int vyska){
	//vraci pole barev pixelu pro pohled hrace
		Color[][] pole=new Color[sirka][vyska];
		for(int i=0; i<sirka; i++){
			for(int l=0; l<vyska; l++){
				if((x+i<0 || y+l <0)|| (y+l>mapa[i].length-1)|| (x+i>mapa.length-1)){
					pole[i][l]=kamen;
				}else {pole[i][l]=mapa[x+i][y+l].barva;}
			}
		}
		return pole;
    }
    //fx pro nedostatek energie
    public Color[][] getStatic(int sirka, int vyska){
		Color[][] pole=new Color[sirka][vyska];
		for(int i=0; i<sirka; i++){
			for(int l=0; l<vyska; l++){
			pole[i][l]=staticBarva();
			}
		}
		return pole;
    }

    //zobrazeni pri novem kole
    public Color[][] getLoad(int sirka, int vyska){
		Color[][] pole=new Color[sirka][vyska];
		for(int i=0; i<sirka; i++){
			for(int l=0; l<vyska; l++){
				pole[i][l]=load;
			}
		}
		return pole;
     }

    private Color staticBarva(){
		return new Color((int) Math.round(Math.random()*255),(int) Math.round(Math.random()*255),(int) Math.round(Math.random()*255));
    }
}